#include <stdexcept>
#include <iostream>
#include <sstream>
#include <memory>
#include <deque>
#include <stdlib.h>
#include <signal.h>
#include <stdio.h>
#include <algorithm>
#include "TROOT.h"
#include "TApplication.h"
#include "TMath.h"
#include "TString.h"
#include "TFile.h"
#include "TObjArray.h"
#include "TStopwatch.h"
#include "TDatabasePDG.h"
#include "TParticlePDG.h"
#include "TLorentzVector.h"
#include "TH1D.h"

#include "modules/Delphes.h"
#include "classes/DelphesStream.h"
#include "classes/DelphesClasses.h"
#include "classes/DelphesFactory.h"
#include "classes/DelphesModule.h"
#include "classes/DelphesFormula.h"

#include "ExRootAnalysis/ExRootTreeWriter.h"
#include "ExRootAnalysis/ExRootTreeBranch.h"
#include "ExRootAnalysis/ExRootProgressBar.h"
#include "ExRootAnalysis/ExRootResult.h"
#include "ExRootAnalysis/ExRootFilter.h"
#include "ExRootAnalysis/ExRootClassifier.h"
#include "ExRootAnalysis/ExRootTask.h"
#include "ExRootAnalysis/ExRootConfReader.h"

#include "promc/ProMCBook.h"


extern vector<string> getfiles(char * directory);


using namespace std;

//---------------------------------------------------------------------------

void ConvertInput(ProMCEvent &event, double kEV, double kLe, DelphesFactory *factory,
                  TObjArray *allParticleOutputArray, TObjArray *stableParticleOutputArray, TObjArray *partonOutputArray,
                  TStopwatch *readStopWatch, TStopwatch *procStopWatch)
{
	Int_t i;

	ProMCEvent_Event *mutableEvent;
	ProMCEvent_Particles *mutableParticles;

	//HepMCEvent *element;
	Candidate *candidate;
	TDatabasePDG *pdg;
	TParticlePDG *pdgParticle;
	Int_t pdgCode;

	Int_t pid, status;
	Double_t px, py, pz, mass;
	Double_t x, y, z, t;

	pdg = TDatabasePDG::Instance();

	// event information
	mutableEvent = event.mutable_event();
	mutableParticles = event.mutable_particles();

	for(i = 0; i < mutableParticles->pdg_id_size(); ++i)
	{
		pid = mutableParticles->pdg_id(i);
		status = mutableParticles->status(i);
		px = mutableParticles->px(i)/kEV; 
                py = mutableParticles->py(i)/kEV; 
                pz = mutableParticles->pz(i)/kEV; 
                mass = mutableParticles->mass(i)/kEV;
		x = mutableParticles->x(i)/kLe; 
                y = mutableParticles->y(i)/kLe; 
                z = mutableParticles->z(i)/kLe; 
                t = mutableParticles->t(i)/kLe;

		candidate = factory->NewCandidate();

		candidate->PID = pid;
		pdgCode = TMath::Abs(candidate->PID);

		candidate->Status = status;

		candidate->M1 = mutableParticles->mother1(i);
		candidate->M2 = mutableParticles->mother2(i);

		candidate->D1 = mutableParticles->daughter1(i);
		candidate->D2 = mutableParticles->daughter2(i);

		pdgParticle = pdg->GetParticle(pid);
		candidate->Charge = pdgParticle ? Int_t(pdgParticle->Charge()/3.0) : -999;
		candidate->Mass = mass;

		candidate->Momentum.SetXYZM(px, py, pz, mass);

		candidate->Position.SetXYZT(x, y, z, t);

		allParticleOutputArray->Add(candidate);

		if(!pdgParticle) continue;

		if(status == 1)
		{
			stableParticleOutputArray->Add(candidate);
		}
		else if(pdgCode <= 5 || pdgCode == 21 || pdgCode == 15)
		{
			partonOutputArray->Add(candidate);
		}
	}
}

