#!/bin/bash
#
# Configuration script to build ProMC
# S.Chekanov. ANL 
# May 2013
echo "Building FastSim.."
# specify compilers here (do not change this part!) 
CXX="g++"
CC="gcc"
AR="ar"
XMAKE="make"


# some cleaning..
unset PROMC
rm -f config.*

export FASTSIM_DIR="$( cd "$( echo "${BASH_SOURCE[0]%/*}" )" && pwd )"
CURRENT_DIR=`pwd`

ARCH0="`uname`"
if [ "$ARCH0" != "Linux" ]
then
echo " Error: operating system=$ARCH0, this is not Linux! Stop";
exit 1;
fi
echo " * ok, platform is $ARCH0"



if [ -n "${ROOTSYS+1}" ]; then
  echo " * ok, ROOTSYS is set to $ROOTSYS"
else
  echo "ROOTSYS not found. ROOT is not installed?. Exit!" 
  exit 1
fi

# now check for Java
if which java >/dev/null; then
    _java=java
    echo " * ok, JAVA  was found"
elif [[ -n "$JAVA_HOME" ]] && [[ -x "$JAVA_HOME/bin/java" ]];  then
    _java="$JAVA_HOME/bin/java"
     echo "* ok, JAVA  was found in JAVA_HOME=$JAVA_HOME"
else
    echo "No java detected! Please install it from https://java.com/download"; exit 0
fi

version="1.6"
if [[ "$_java" ]]; then
    version=$("$_java" -version 2>&1 | awk -F '"' '/version/ {print $2}')
    if [[ "$version" < "1.7" ]]; then
        echo "Java version is less than 1.7. Please update it."; exit 0;
    fi
fi


fcheck=`type -p $CC`
if test -x "$fcheck";
then
    echo " * ok, $fcheck -> $CC found" 
else
    echo "Error: $CC does not exist"
    exit 0;
fi


fcheck=`type -p $CXX`
if test -x "$fcheck";
then
    echo " * ok, $fcheck -> $CXX found"
else
    echo "Error: $CXX does not exist"
    exit 0;
fi


ZLIB="/usr/include/zlib.h"
if [ -f $FILE ]
then
    echo " * ok, $ZLIB -> is found"
else
   echo "Error: zlib-dev does not exist"
   echo "Try: yum install zlib-devel (or) sudo apt-get install zlib1g-dev (ubuntu)" 
   exit 0;
fi


# check make
fcheck=`type -p $XMAKE`
if test -x "$fcheck";
then
    echo " * ok, $fcheck -> GNU $XMAKE found"
else
    echo "Error: $XMAKE does not exist"
    exit 0;
fi


fcheck=`type -p $AR`
if test -x "$fcheck";
then
    echo " * ok, $fcheck -> $AR  found"
else
    echo "Error: $AR does not exist"
    exit 0;
fi


fcheck=`type -p ranlib`
if test -x "$fcheck";
then
    echo " * ok, $fcheck -> ranlib found"
else
    echo "Error: ranlib does not exist"
    exit 0;
fi


fcheck=`type -p python`
PYTHON_OK=`python -c 'import sys; print (sys.version_info >= (2, 5) and "1" or "0")'`
if [ "$PYTHON_OK" = '0' ]; then
    echo "  Note: Python version is too old. Should be >2.4. PYTHON is not used"
    #exit 0;
else
    echo " * ok, $fcheck -> PYTHON is found"
fi

ARCH=`arch`
echo "-->Success. Library for $ARCH"



fcheck=`type -p protoc`
if test -x "$fcheck";
then
    PRBUF=`protoc --version`
    echo "Warning: Protocol Buffers $PRBUF installed in $fcheck"
    echo "         ProMC uses an internal Protocol Buffers. Continue."
fi


echo "Preparing..."
chmod -R 755 $CURRENT_DIR


#
# User-changeable part, experts  -----------------------------
CPP_FLAGS=" -ansi -fPIC -Wall -c"
CFLAGS="${CFLAGS_OPT}"
CXXFLAGS="${CXXFLAGS_OPT}"


CONF="$CURRENT_DIR/cmt/config.mk"
echo "Creating $CONF"
rm -f $CONF 
echo SHELL=/bin/sh > $CONF 
echo DIR_SHARE=$CURRENT_DIR/ProMC/share >> $CONF 
echo DIR_CBOOK=$CURRENT_DIR/ProMC/cbook >> $CONF
echo DIR_CURRENT=$CURRENT_DIR >> $CONF
echo AR=${AR} >> $CONF
echo ARCH=${ARCH} >> $CONF
echo CXX=${CXX} >> $CONF
echo CPP_COMP=${CXX} >> $CONF
echo CPP_FLAGS=${CPP_FLAGS} >> $CONF
echo CC=${CC} >> $CONF


CONF="$CURRENT_DIR/cmt/config.sh"
echo "Creating $CONF"
rm -f $CONF
echo "#!/bin/bash" > $CONF
echo "DIR_SHARE=$CURRENT_DIR/share" >> $CONF
echo "DIR_CBOOK=$CURRENT_DIR/cbook" >> $CONF 
echo "DIR_CURRENT=$CURRENT_DIR" >> $CONF 
echo "AR=${AR}" >> $CONF 
echo "ARCH=${ARCH}" >> $CONF
echo "CXX=${CXX}" >> $CONF
echo "CPP_COMP=${CXX}" >> $CONF
echo "CPP_FLAGS=\"${CPP_FLAGS}\"" >> $CONF
echo "CC=${CC}" >> $CONF

echo "Building ProMC.."
cd $CURRENT_DIR/ProMC
./build.sh
./install.sh $FASTSIM_DIR/lib
cd $CURRENT_DIR/ 

echo "Setup ProMC"
source ./lib/promc/setup.sh
source ./hs-toolkit/setup.sh

echo "Setup Delphes.."
export DELPHES=$FASTSIM_DIR/Delphes
cd $DELPHES
./configure
make DelphesProMC
make libDelphes.so
#cd $FASTSIM_DIR/cmt; 
#mkdir -p $FASTSIM_DIR/lib/delphes
#make
cd $CURRENT_DIR
 
# 
echo "All done.."
